'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = sagaMiddleware;

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

var SAGA_ARGUMENT_ERROR = exports.SAGA_ARGUMENT_ERROR = "Saga must be a Generator function";

function isGenerator(fn) {
  return fn.constructor.name === 'GeneratorFunction';
}

function sagaMiddleware(saga) {
  if (!isGenerator(saga)) throw new Error(SAGA_ARGUMENT_ERROR);

  return function (_ref) {
    var getState = _ref.getState;
    var dispatch = _ref.dispatch;
    return function (next) {
      return function (action) {

        // hit the reducer
        var result = next(action);

        // hit the saga
        var generator = saga(getState, action);
        iterate(generator);

        return result;

        function iterate(generator) {

          step();

          function step(arg, isError) {
            var _ref2 = isError ? generator.throw(arg) : generator.next(arg);

            var effect = _ref2.value;
            var done = _ref2.done;

            // retreives next action/effect

            if (!done) {
              var response = undefined;
              if (typeof effect === 'function') {
                response = effect();
              } else if (Array.isArray(effect) && typeof effect[0] === 'function') {
                response = effect[0].apply(effect, _toConsumableArray(effect.slice(1)));
              } else {
                response = dispatch(effect);
              }

              Promise.resolve(response).then(step, function (err) {
                return step(err, true);
              });
            }
          }
        }
      };
    };
  };
}